@@ -2,8 +2,7 @@
 
 from .client import ONVIFClient
 from .operator import ONVIFOperator, CacheMode
-from .utils import ONVIFWSDL, ONVIFOperationException
-from .utils.zeep import apply_patch, remove_patch, is_patched
+from .utils import ONVIFWSDL, ONVIFOperationException, ONVIFErrorHandler, ZeepPatcher
 
 __all__ = [
     "ONVIFClient",
@@ -11,7 +10,6 @@ __all__ = [
     "CacheMode",
     "ONVIFWSDL",
     "ONVIFOperationException",
-    "apply_patch",
-    "remove_patch",
-    "is_patched",
+    "ONVIFErrorHandler",
+    "ZeepPatcher",
 ]
